/*
 * muppetcam - uploading webcam viewer
 * by muppet <scott@asofyet.org>, 20-21 june 2002
 *
 * camera.c - base class for camera driver objects
 */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>

#include "camera.h"


/*
 * initialize the members of the Camera base class.  this does NOT allocate
 * the object --- this is meant to be called from the subclass constructor.
 */
void camera_construct ( Camera * cam, int handle, char * name, int width, int height, int depth )
{
	cam->handle = handle;
	if ( name )
		cam->name = g_strdup ( name );
	else 
		name = NULL;
	cam->width = width;
	cam->height = height;
	cam->depth = depth;
	cam->image = (byte*) malloc ( width * height * depth );
	cam->ncontrols = 0;
	cam->control = NULL;
}

void camera_destroy ( Camera * cam )
{
	int i;
	/* call subclass' destructor */
	if ( cam->destroy ) cam->destroy ( cam );
	if ( cam->name ) free ( cam->name );
	if ( cam->image ) free ( cam->image );
	for ( i = 0 ; i < cam->ncontrols ; i++ )
		cam_control_destroy ( cam->control[i] );
	free ( cam );
}


int  camera_snap  ( Camera * cam )
{
	return cam->snap ( cam );
}


/*************************************/


CamControl * cam_control_new ( CamControlType  type, 
                               char           * name, 
                               GtkAdjustment  * adj )
{
	CamControl * cc = (CamControl*) malloc ( sizeof (CamControl) );
	cc->type = type;
	cc->name = g_strdup ( name );
	cc->adj = adj;
	gtk_object_ref ( GTK_OBJECT(adj) );
	return cc;
}

void cam_control_destroy ( CamControl * cc )
{
	free ( cc->name );
	gtk_object_unref ( GTK_OBJECT( cc->adj ) );
	free ( cc );
}


/*
 * creates a composite widget and returns the toplevel one, ready to be
 * gtk_widget_show()n.
 */
GtkWidget * cam_control_create_widget ( CamControl * cc )
{
	GtkWidget * hbox;
	GtkWidget * label;
	GtkWidget * slider;
	
	if ( cc->type != CAMCTRL_SLIDER ) {
		fprintf(stderr,"non-slider CamControls aren't implemented!\n");
		return NULL;
	}
	
	hbox = gtk_hbox_new ( FALSE, 0 );
	label = gtk_label_new ( cc->name );
	slider = gtk_hscale_new ( cc->adj );
	
	gtk_box_pack_start ( GTK_BOX(hbox), label, FALSE, FALSE, 0 );
	gtk_box_pack_start ( GTK_BOX(hbox), slider, TRUE, TRUE, 0 );

	gtk_widget_show ( label );
	gtk_widget_show ( slider );
	
	return hbox;
}


/*************************************/
