/*
 * muppetcam - uploading webcam viewer
 * by muppet <scott@asofyet.org>, 20-21 june 2002
 *
 * camera.h - declarations for Camera and CamControl
 *
 * Camera is a camera driver base class, which contains a captured image
 * buffer, a list (actually an array) of CamControls, and some basic methods
 * like snap and destroy, which should be fleshed out by descendent classes.
 *
 * Don't instantiate a plain old Camera directly, always instantiate a 
 * subclass.  Otherwise, you should be able to use the base class for 
 * everything else (which admittedly isn't much).
 */

/*************************************/

#include <gtk/gtk.h>	/* for GtkAdjustment and GtkWidget */

/*
 * CamControl - generic control value, and something to create a widget 
 *              for it.  not really specific to cameras, but that's what
 *              i use it for.
 *
 *              think of this as a named value.
 */


/*
 * control type -- determines the kind of value used and the widgets that 
 * will be created.
 */
typedef enum {
	CAMCTRL_SLIDER,
	CAMCTRL_BOOLEAN,	/* not implemented */
	CAMCTRL_STRING,		/* not implemented */
	CAMCTRL_SPINNER		/* not implemented */
} CamControlType;



typedef struct {
	CamControlType   type;
	char           * name;
	GtkAdjustment  * adj;
} CamControl;


CamControl * cam_control_new           ( CamControlType   type, 
                                         char           * name, 
                                         GtkAdjustment  * adj );

void         cam_control_destroy       ( CamControl     * cc );

GtkWidget  * cam_control_create_widget ( CamControl     * cc );

/*************************************/


typedef unsigned char byte;


typedef struct _Camera Camera;

typedef int (*CameraFunc) ( Camera * );

struct _Camera {
	/* how to get a hold of me... */
	int           handle;
	char        * name;
	
	/* the image */
	byte        * image;		/* image data */
	int           width;		/* in pixels */
	int           height;		/* in pixels */
	int           depth;		/* bytes per pixel */

	/* controls */
	int           ncontrols;
	CamControl ** control;
	
	/* class support stuff.... */
	CameraFunc    snap;
	CameraFunc    destroy;
};

#define CAMERA(cam)				((Camera*)cam)
#define camera_is_color(cam)			(cam->depth == 3)
#define camera_control_name(cam, which)		(cam->control[which]->name)
#define camera_control_value(cam, which)	(cam->control[which]->adj->value)

/*
 * call this to take a picture.  if it returns true, you should have a new
 * image in cam->image.
 */
int  camera_snap    ( Camera * cam );

/*
 * call to release a Camera object.  call this one, NOT the subclass'.
 */
void camera_destroy ( Camera * cam );



/* private, should be used only by subclass constructors */
void camera_construct ( Camera * cam, 
                        int      handle, 
                        char   * name, 
                        int      width, 
                        int      height, 
                        int      depth );

