/*
 * muppetcam - uploading webcam viewer
 * by muppet <scott@asofyet.org>, 20-21 june 2002
 *
 * serverdialog.h - modeless dialog that allows the user to set the hostname
 *                  and filename for uploading captured images.
 */
#include <stdio.h>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

#include "camera.h"
#include "muppetcam.h"
#include "serverdialog.h"


static void server_close            ( GtkWidget    * button, 
                                      ServerDialog * sd );
static void server_delete_event     ( GtkWidget    * window, 
                                      GdkEventAny  * event, 
                                      ServerDialog * sd );
static int  server_key_press_event  ( GtkWidget    * window, 
                                      GdkEventKey  * event, 
                                      ServerDialog * sd );
static void server_ok               ( GtkWidget    * button, 
                                      ServerDialog * sd );



void server_close ( GtkWidget * button, ServerDialog * sd )
{
	gtk_widget_destroy ( sd->mc->server_dialog );
	sd->mc->server_dialog = NULL;
	free ( sd );
}

void server_delete_event ( GtkWidget    * window, 
                           GdkEventAny  * event, 
                           ServerDialog * sd )
{
	server_close ( NULL, sd );
}

int server_key_press_event ( GtkWidget    * window, 
                             GdkEventKey  * event, 
                             ServerDialog * sd )
{
	if ( event->keyval == GDK_Escape ) {
		server_close ( NULL, sd );
		return 1;
	}
		
	return 0;
}

void server_ok ( GtkWidget * button, ServerDialog * sd )
{
	if ( sd->mc->hostname ) free ( sd->mc->hostname );
	if ( sd->mc->filename ) free ( sd->mc->filename );
	
	sd->mc->hostname = g_strdup ( gtk_entry_get_text ( sd->hostname ) );
	sd->mc->filename = g_strdup ( gtk_entry_get_text ( sd->filename ) );

	server_close ( NULL, sd );
}


void server_clicked ( GtkWidget * widget, MuppetCamWindow * mc )
{
	ServerDialog * sd;
	GtkWidget * dlg;
	GtkWidget * button;
	GtkWidget * table;
	GtkWidget * label;

	if ( mc->server_dialog ) {
		gdk_window_raise ( GTK_WIDGET(mc->server_dialog)->window );
		return;
	}
	
	sd = (ServerDialog*) calloc ( 1, sizeof (ServerDialog) );
	sd->mc = mc;
	
	dlg = gtk_dialog_new ();
	gtk_window_set_title ( GTK_WINDOW(dlg), "Configure Server" );

	label = gtk_label_new ( "Enter the destination host name"
				" and the destination file path from the"
				" host's login root.  The user name and"
				" password for this server must be entered"
				" in your ~/.netrc file.  See ftp(1) for"
				" details.");
	gtk_label_set_line_wrap ( GTK_LABEL(label), 1 );
	gtk_box_pack_start ( GTK_BOX( GTK_DIALOG(dlg)->vbox ), 
			label, FALSE, FALSE, 10 );
	
	sd->hostname = (GtkEntry*) gtk_entry_new ();
	sd->filename = (GtkEntry*) gtk_entry_new ();

	gtk_entry_set_text ( sd->hostname, mc->hostname ? mc->hostname : "" );
	gtk_entry_set_text ( sd->filename, mc->filename ? mc->filename : "" );

	table = gtk_table_new ( 4, 2, FALSE );
	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dlg)->vbox), table, 
			FALSE, FALSE, 3 );

	gtk_table_attach_defaults ( GTK_TABLE(table),
			gtk_label_new ( "Hostname: " ), 0, 1, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table),
			GTK_WIDGET(sd->hostname), 1, 2, 0, 1 );
	gtk_table_attach_defaults ( GTK_TABLE(table),
			gtk_label_new ( "Filename: " ), 0, 1, 3, 4 );
	gtk_table_attach_defaults ( GTK_TABLE(table),
			GTK_WIDGET(sd->filename), 1, 2, 3, 4 );

	gtk_container_set_border_width 
		( GTK_CONTAINER( GTK_DIALOG(dlg)->vbox ), 6 );
	
	button = gtk_button_new_with_label ( "Ok" );
	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dlg)->action_area), 
			button, TRUE, TRUE, 10 );
	gtk_signal_connect ( GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(server_ok), sd );
	button = gtk_button_new_with_label ( "Cancel" );
	gtk_box_pack_start ( GTK_BOX(GTK_DIALOG(dlg)->action_area), 
			button, TRUE, TRUE, 10 );
	gtk_signal_connect ( GTK_OBJECT(button), "clicked",
			GTK_SIGNAL_FUNC(server_close), sd );
	gtk_signal_connect ( GTK_OBJECT(dlg), "delete_event",
			GTK_SIGNAL_FUNC(server_delete_event), sd );
	gtk_signal_connect ( GTK_OBJECT(dlg), "key_press_event",
			GTK_SIGNAL_FUNC(server_key_press_event), sd );
	
	gtk_widget_show_all ( dlg );
	mc->server_dialog = dlg;
}

